@@ -49,8 +49,8 @@ def addV1System(snmpEngine, securityName, communityName,
 
     if contextEngineId is None:
         contextEngineId = snmpEngineID.syntax
-    if contextName is not None:
-        contextName = communityName
+    if contextName is None:
+        contextName = ''
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
         ((snmpCommunityEntry.name + (8,) + tblIdx, 'destroy'),)
@@ -256,10 +256,10 @@ def addTargetAddr(
         snmpEngine, addrName
         )
     
-    if transportDomain == snmpUDPDomain:
+    if transportDomain[:len(snmpUDPDomain)] == snmpUDPDomain:
         SnmpUDPAddress, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('SNMPv2-TM', 'SnmpUDPAddress')
         transportAddress = SnmpUDPAddress(transportAddress)
-    elif transportDomain == snmpUDP6Domain:
+    elif transportDomain[:len(snmpUDP6Domain)] == snmpUDP6Domain:
         TransportAddressIPv6, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('TRANSPORT-ADDRESS-MIB', 'TransportAddressIPv6')
         transportAddress = TransportAddressIPv6(transportAddress)
 
@@ -428,7 +428,8 @@ def __cookVacmUserInfo(snmpEngine, securityModel, securityName, securityLevel):
              'r' + groupName, 'w' + groupName, 'n' + groupName )
 
 def addVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                readSubTree=(), writeSubTree=(), notifySubTree=()):
+                readSubTree=(), writeSubTree=(), notifySubTree=(),
+                contextName=''):
     ( groupName, securityLevel,
       readView, writeView, notifyView ) = __cookVacmUserInfo(
         snmpEngine, securityModel, securityName, securityLevel,
@@ -437,7 +438,7 @@ def addVacmUser(snmpEngine, securityModel, securityName, securityLevel,
         snmpEngine, groupName, securityModel, securityName
         )
     addVacmAccess(
-        snmpEngine, groupName, '', securityModel, securityLevel, 1,
+        snmpEngine, groupName, contextName, securityModel, securityLevel, 1,
         readView, writeView, notifyView
         )
     if readSubTree:
