@@ -3,7 +3,7 @@ from pysnmp.smi.indices import OidOrderedDict
 from pysnmp.smi import exval, error
 from pysnmp.proto import rfc1902
 from pyasn1.type import constraint
-from pyasn1.error import ValueConstraintError
+from pyasn1.error import ValueConstraintError, PyAsn1Error
 from pysnmp import debug
 
 ( Integer, ObjectIdentifier, Null ) = mibBuilder.importSymbols("ASN1", "Integer", "ObjectIdentifier", "Null")
@@ -90,7 +90,7 @@ MODULE-IDENTITY\n\
      self.getOrganization(),
      self.getContactInfo(),
      self.getDescription(),
-     string.join(map(lambda x: "REVISON \"%s\"\n" % x, self.getRevisions())))
+     string.join(map(lambda x: "REVISION \"%s\"\n" % x, self.getRevisions())))
 
 class ObjectIdentity(MibNode):
     def getStatus(self):
@@ -300,7 +300,7 @@ class MibTree(ObjectType):
                    self.maxAccess != 'readonly' and \
                    self.maxAccess != 'readwrite' and \
                    self.maxAccess != 'readcreate' or \
-                   acFun and acFun(name, idx, 'read', acCtx):
+                   acFun and acFun(name, self.syntax, idx, 'read', acCtx):
                 raise error.NoAccessError(idx=idx, name=name)
         else:
             try:
@@ -354,7 +354,7 @@ class MibTree(ObjectType):
             if acFun and \
                    self.maxAccess != 'readwrite' and \
                    self.maxAccess != 'readcreate' or \
-                   acFun and acFun(name, idx, 'write', acCtx):
+                   acFun and acFun(name, self.syntax, idx, 'write', acCtx):
                 raise error.NotWritableError(idx=idx, name=name)
         else:
             node = self.getBranch(name, idx)
@@ -387,7 +387,7 @@ class MibScalar(MibTree):
                self.maxAccess != 'readonly' and \
                self.maxAccess != 'readwrite' and \
                self.maxAccess != 'readcreate' or \
-               acFun and acFun(name, idx, 'read', acCtx):
+               acFun and acFun(name, self.syntax, idx, 'read', acCtx):
             raise error.NoAccessError(idx=idx, name=name)
     
     # Two-phase commit implementation
@@ -401,7 +401,7 @@ class MibScalar(MibTree):
         if acFun and \
                self.maxAccess != 'readwrite' and \
                self.maxAccess != 'readcreate' or \
-               acFun and acFun(name, idx, 'write', acCtx):
+               acFun and acFun(name, self.syntax, idx, 'write', acCtx):
             raise error.NotWritableError(idx=idx, name=name)
 
 class MibScalarInstance(MibTree):
@@ -433,7 +433,7 @@ class MibScalarInstance(MibTree):
         # Return current variable (name, value). This is the only API method
         # capable of returning anything!
         if name == self.name:
-            debug.logger & debug.flagIns and debug.logger('readGet: %s=%s' % (self.name, self.syntax))
+            debug.logger & debug.flagIns and debug.logger('readGet: %s=%s' % (self.name, repr(self.syntax)))
             return self.name, self.syntax.clone()
         else:
             raise error.NoSuchObjectError(idx=idx, name=name)
@@ -460,7 +460,7 @@ class MibScalarInstance(MibTree):
         self.syntax = self.__newSyntax
         
     def writeCleanup(self, name, val, idx, (acFun, acCtx)):
-        debug.logger & debug.flagIns and debug.logger('writeCleanup: %s=%s' % (name, val))
+        debug.logger & debug.flagIns and debug.logger('writeCleanup: %s=%s' % (name, repr(val)))
         # Drop previous value
         self.__newSyntax = self.__oldSyntax = None
     
@@ -485,12 +485,12 @@ class MibScalarInstance(MibTree):
         if val is not None:
             self.writeCommit(name, val, idx, (acFun, acCtx))
     def createCleanup(self, name, val, idx, (acFun, acCtx)):
-        debug.logger & debug.flagIns and debug.logger('createCleanup: %s=%s' % (name, val))
+        debug.logger & debug.flagIns and debug.logger('createCleanup: %s=%s' % (name, repr(val)))
         if val is not None:
             self.writeCleanup(name, val, idx, (acFun, acCtx))
     def createUndo(self, name, val, idx, (acFun, acCtx)):
         if val is not None:
-            self.writeCleanup(name, val, idx, (acFun, acCtx))
+            self.writeUndo(name, val, idx, (acFun, acCtx))
 
     # Destroy operation
 
@@ -515,6 +515,15 @@ class MibTableColumn(MibScalar):
         MibScalar.__init__(self, name, syntax)
         self.__createdInstances = {}; self.__destroyedInstances = {}
         self.__rowOpWanted = {}
+        self.__valIdx = {} # column instance value to OID index
+        self.__valIdxId = -1
+        
+    # No branches here, terminal OIDs only
+    def getBranch(self, name, idx):
+        if len(self.name) < len(name):
+            if self._vars.has_key(name):
+                return self._vars[name]
+        raise error.NoSuchObjectError(name=name, idx=idx)
 
     def getNode(self, name, idx=None):
         try:
@@ -531,6 +540,36 @@ class MibTableColumn(MibScalar):
     def setProtoInstance(self, protoInstance):
         self.protoInstance = protoInstance
 
+    # Value to instances map
+
+    def getNextNodeWithValue(self, name, value):
+        if not self.__valIdx or self.__valIdxId != self.branchVersionId:
+            # Build a value->oid index
+            node = self
+            while 1:
+                try:
+                    node = self.getNextNode(node.name)
+                except error.NoSuchInstanceError:
+                    break
+                if not self.__valIdx.has_key(node.syntax):
+                    self.__valIdx[node.syntax] = OidOrderedDict()
+                self.__valIdx[node.syntax][node.name] = 1
+
+            debug.logger & debug.flagIns and debug.logger('value index rebuilt at %s, %s entries' % (self.name,len(self.__valIdx)))
+
+            # Sync to tree version
+            self.__valIdxId = self.branchVersionId
+
+        if self.__valIdx.has_key(value):
+            try:
+                return self.getNode(
+                    self.__valIdx[value].nextKey(name)
+                    )
+            except KeyError:
+                raise error.NoSuchInstanceError(name=name)
+        else:
+            raise error.NoSuchInstanceError(name=name)
+            
     # Column creation (this should probably be converted into some state
     # machine for clarity). Also, it might be a good idea to inidicate
     # defaulted cols creation in a clearer way than just a val == None.
@@ -543,7 +582,8 @@ class MibTableColumn(MibScalar):
         if acFun and \
                val is not None and \
                self.maxAccess != 'readcreate' or \
-               acFun and acFun(name, idx, 'write', acCtx):
+               acFun and acFun(name, self.syntax, idx, 'write', acCtx):
+            debug.logger & debug.flagACL and debug.logger('createTest: %s=%s %s at %s' % (name, repr(val), self.maxAccess, self.name))
             raise error.NoCreationError(idx=idx, name=name)
         # Create instances if either it does not yet exist (row creation)
         # or a value is passed (multiple OIDs in SET PDU)
@@ -570,6 +610,9 @@ class MibTableColumn(MibScalar):
                           self.__createdInstances[name], self._vars.get(name)
 
     def createCleanup(self, name, val, idx, (acFun, acCtx)):
+        # Drop by-value index
+        self.__valIdx.clear()
+        
         # Drop previous column instance
         if self.__createdInstances.has_key(name):
             if self.__createdInstances[name] is not None:
@@ -584,12 +627,18 @@ class MibTableColumn(MibScalar):
         # Set back previous column instance, drop the new one
         if self.__createdInstances.has_key(name):
             self._vars[name] = self.__createdInstances[name]
-            del self.__createdInstances[name]            
+            del self.__createdInstances[name]
             # Remove new instance on rollback
             if self._vars[name] is None:
                 del self._vars[name]
             else:
-                self._vars[name].createUndo(name, val, idx, (acFun, acCtx))
+                # Catch half-created instances (hackerish)
+                try:
+                    self._vars[name] == 0
+                except PyAsn1Error:
+                    del self._vars[name]
+                else:
+                    self._vars[name].createUndo(name, val, idx, (acFun, acCtx))
                 
     # Column destruction
         
@@ -602,7 +651,7 @@ class MibTableColumn(MibScalar):
         if acFun and \
                val is not None and \
                self.maxAccess != 'readcreate' or \
-               acFun and acFun(name, idx, 'write', acCtx):
+               acFun and acFun(name, self.syntax, idx, 'write', acCtx):
             raise error.NoAccessError(idx=idx, name=name)
         self._vars[name].destroyTest(
             name, val, idx, (acFun, acCtx)
@@ -618,12 +667,15 @@ class MibTableColumn(MibScalar):
             del self._vars[name]
         
     def destroyCleanup(self, name, val, idx, (acFun, acCtx)):
+        # Drop by-value index
+        self.__valIdx.clear()
+        
         # Drop instance copy
         if self.__destroyedInstances.has_key(name):
             self.__destroyedInstances[name].destroyCleanup(
                 name, val, idx, (acFun, acCtx)
                 )
-            debug.logger & debug.flagIns and debug.logger('destroyCleanup: %s=%s' % (name, val))
+            debug.logger & debug.flagIns and debug.logger('destroyCleanup: %s=%s' % (name, repr(val)))
             del self.__destroyedInstances[name]
             
     def destroyUndo(self, name, val, idx, (acFun, acCtx)):
@@ -652,7 +704,7 @@ class MibTableColumn(MibScalar):
             self.__rowOpWanted[name] = error.RowDestructionWanted()
             self.destroyTest(name, val, idx, (acFun, acCtx))
         if self.__rowOpWanted.has_key(name):
-            debug.logger & debug.flagIns and debug.logger('%s flagged by %s=%s' % (self.__rowOpWanted[name], name, val))
+            debug.logger & debug.flagIns and debug.logger('%s flagged by %s=%s' % (self.__rowOpWanted[name], name, repr(val)))
             raise self.__rowOpWanted[name]
 
     def __delegateWrite(self, subAction, name, val, idx, (acFun, acCtx)):
@@ -678,13 +730,16 @@ class MibTableColumn(MibScalar):
             raise self.__rowOpWanted[name]
 
     def writeCleanup(self, name, val, idx, (acFun, acCtx)):
+        # Drop by-value index
+        self.__valIdx.clear()
+        
         self.__delegateWrite(
             'Cleanup', name, val, idx, (acFun, acCtx)
             )
         if self.__rowOpWanted.has_key(name):
             e = self.__rowOpWanted[name]
             del self.__rowOpWanted[name]
-            debug.logger & debug.flagIns and debug.logger('%s dropped by %s=%s' % (e, name, val))
+            debug.logger & debug.flagIns and debug.logger('%s dropped by %s=%s' % (e, name, repr(val)))
             raise e
             
     def writeUndo(self, name, val, idx, (acFun, acCtx)):
@@ -694,7 +749,7 @@ class MibTableColumn(MibScalar):
         if self.__rowOpWanted.has_key(name):
             e = self.__rowOpWanted[name]
             del self.__rowOpWanted[name]
-            debug.logger & debug.flagIns and debug.logger('%s dropped by %s=%s' % (e, name, val))
+            debug.logger & debug.flagIns and debug.logger('%s dropped by %s=%s' % (e, name, repr(val)))
             raise e
 
 class MibTableRow(MibTree):
@@ -869,7 +924,7 @@ class MibTableRow(MibTree):
             else:
                 getattr(var, action)(name + nameSuffix, val, idx,
                                      (acFun, acCtx))
-            debug.logger & debug.flagIns and debug.logger('__manageColumns: action %s name %s suffix %s %svalue %s' % (action, name, nameSuffix, indexVals.has_key(name) and "index " or "", indexVals.get(name, val)))
+            debug.logger & debug.flagIns and debug.logger('__manageColumns: action %s name %s suffix %s %svalue %s' % (action, name, nameSuffix, indexVals.has_key(name) and "index " or "", repr(indexVals.get(name, val))))
 
     def __delegate(self, subAction, name, val, idx, (acFun, acCtx)):
         # Relay operation request to column, expect row operation request.
